# -*- coding: utf-8; -*-
################################################################################
#
#  wuttaweb -- Web App for Wutta Framework
#  Copyright © 2024 Lance Edgar
#
#  This file is part of Wutta Framework.
#
#  Wutta Framework is free software: you can redistribute it and/or modify it
#  under the terms of the GNU General Public License as published by the Free
#  Software Foundation, either version 3 of the License, or (at your option) any
#  later version.
#
#  Wutta Framework is distributed in the hope that it will be useful, but
#  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
#  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
#  more details.
#
#  You should have received a copy of the GNU General Public License along with
#  Wutta Framework.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
"""
Auth Views
"""

import colander
from deform.widget import TextInputWidget, PasswordWidget

from wuttaweb.views import View
from wuttaweb.db import Session
from wuttaweb.auth import login_user, logout_user


class AuthView(View):
    """
    Auth views shared by all apps.
    """

    def login(self, session=None):
        """
        View for user login.

        This view shows the login form, and handles its submission.
        Upon successful login, user is redirected to home page.

        * route: ``login``
        * template: ``/login.mako``
        """
        auth = self.app.get_auth_handler()

        # TODO: should call request.get_referrer()
        referrer = self.request.route_url('home')

        # redirect if already logged in
        if self.request.user:
            self.request.session.flash(f"{self.request.user} is already logged in", 'error')
            return self.redirect(referrer)

        form = self.make_form(schema=self.login_make_schema(),
                              align_buttons_right=True,
                              show_button_reset=True,
                              button_label_submit="Login",
                              button_icon_submit='user')

        # TODO
        # form.show_cancel = False

        # validate basic form data (sanity check)
        data = form.validate()
        if data:

            # truly validate user credentials
            session = session or Session()
            user = auth.authenticate_user(session, data['username'], data['password'])
            if user:

                # okay now they're truly logged in
                headers = login_user(self.request, user)
                return self.redirect(referrer, headers=headers)

            else:
                self.request.session.flash("Invalid user credentials", 'error')

        return {
            'index_title': self.app.get_title(),
            'form': form,
            # TODO
            # 'referrer': referrer,
        }

    def login_make_schema(self):
        schema = colander.Schema()

        # nb. we must explicitly declare the widgets in order to also
        # specify the ref attribute.  this is needed for autofocus and
        # keydown behavior for login form.

        schema.add(colander.SchemaNode(
            colander.String(),
            name='username',
            widget=TextInputWidget(attributes={
                'ref': 'username',
            })))

        schema.add(colander.SchemaNode(
            colander.String(),
            name='password',
            widget=PasswordWidget(attributes={
                'ref': 'password',
            })))

        return schema

    def logout(self):
        """
        View for user logout.

        This deletes/invalidates the current user session and then
        redirects to the login page.

        Note that a simple GET is sufficient; POST is not required.

        * route: ``logout``
        * template: n/a
        """
        # truly logout the user
        headers = logout_user(self.request)

        # TODO
        # # redirect to home page after logout, if so configured
        # if self.config.get_bool('wuttaweb.home_after_logout', default=False):
        #     return self.redirect(self.request.route_url('home'), headers=headers)

        # otherwise redirect to referrer, with 'login' page as fallback
        # TODO: should call request.get_referrer()
        # referrer = self.request.get_referrer(default=self.request.route_url('login'))
        referrer = self.request.route_url('login')
        return self.redirect(referrer, headers=headers)

    @classmethod
    def defaults(cls, config):
        cls._auth_defaults(config)

    @classmethod
    def _auth_defaults(cls, config):

        # login
        config.add_route('login', '/login')
        config.add_view(cls, attr='login',
                        route_name='login',
                        renderer='/login.mako')

        # logout
        config.add_route('logout', '/logout')
        config.add_view(cls, attr='logout',
                        route_name='logout')


def defaults(config, **kwargs):
    base = globals()

    AuthView = kwargs.get('AuthView', base['AuthView'])
    AuthView.defaults(config)


def includeme(config):
    defaults(config)
